package com.supwisdom.dlpay

import com.supwisdom.dlpay.framework.core.JwtConfig
import com.supwisdom.dlpay.framework.core.JwtTokenUtil
import com.supwisdom.dlpay.framework.dao.OperRoleDao
import com.supwisdom.dlpay.framework.dao.OperatorDao
import com.supwisdom.dlpay.framework.security.ValidateCodeSecurityConfig
import com.supwisdom.dlpay.framework.service.OperatorDetailService
import com.supwisdom.dlpay.framework.util.StringUtil
import org.jose4j.jwt.consumer.InvalidJwtException
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.context.annotation.Bean
import org.springframework.context.annotation.Configuration
import org.springframework.core.annotation.Order
import org.springframework.http.HttpStatus
import org.springframework.security.authentication.AuthenticationCredentialsNotFoundException
import org.springframework.security.authentication.AuthenticationProvider
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken
import org.springframework.security.authentication.dao.AbstractUserDetailsAuthenticationProvider
import org.springframework.security.authentication.dao.DaoAuthenticationProvider
import org.springframework.security.authentication.jaas.AuthorityGranter
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder
import org.springframework.security.config.annotation.web.builders.HttpSecurity
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter
import org.springframework.security.config.http.SessionCreationPolicy
import org.springframework.security.core.Authentication
import org.springframework.security.core.AuthenticationException
import org.springframework.security.core.GrantedAuthority
import org.springframework.security.core.authority.AuthorityUtils
import org.springframework.security.core.authority.SimpleGrantedAuthority
import org.springframework.security.core.context.SecurityContextHolder
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder
import org.springframework.security.web.authentication.AuthenticationFailureHandler
import org.springframework.security.web.authentication.AuthenticationSuccessHandler
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter
import org.springframework.security.web.authentication.rememberme.JdbcTokenRepositoryImpl
import org.springframework.security.web.util.matcher.AntPathRequestMatcher
import org.springframework.stereotype.Component
import org.springframework.web.filter.OncePerRequestFilter
import javax.servlet.FilterChain
import javax.servlet.http.HttpServletRequest
import javax.servlet.http.HttpServletResponse
import javax.sql.DataSource


class ApiJwtAuthenticationFilter(jwt: JwtTokenUtil) : OncePerRequestFilter() {
    private val jwtUtil = jwt

    override fun doFilterInternal(request: HttpServletRequest, response: HttpServletResponse, filterChain: FilterChain) {
        request.getHeader(jwtUtil.header)?.let { jwt ->
            try {
                val claims = jwtUtil.verifyToken(jwt)
                val auth = UsernamePasswordAuthenticationToken(claims["uid"], null,
                        claims["authorities"]?.map { SimpleGrantedAuthority(it as String) })
                SecurityContextHolder.getContext().authentication = auth
            } catch (e: InvalidJwtException) {
                SecurityContextHolder.clearContext();
                response.sendError(HttpStatus.BAD_REQUEST.value(), e.message)
            }
        }
        filterChain.doFilter(request, response)
    }
}

@EnableWebSecurity
class WebSecurityConfig {


    companion object {
        @Configuration
        @Order(1)
        class ApiWebSecurityConfigurationAdapter : WebSecurityConfigurerAdapter() {

            @Autowired
            lateinit var jwtConfig: JwtConfig

            override fun configure(http: HttpSecurity) {
                // 设置 API 访问权限管理
                http.sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                        .and()
                        .addFilterAfter(ApiJwtAuthenticationFilter(JwtTokenUtil(jwtConfig)),
                                UsernamePasswordAuthenticationFilter::class.java)
                        .antMatcher("/api/**")
                        .authorizeRequests()
                        .antMatchers("/api/auth/**").permitAll()
                        .antMatchers("/api/notify/**").permitAll()
                        .antMatchers("/api/common/**").hasAnyRole("THIRD_COMMON", "THIRD_ADMIN")
                        .antMatchers("/api/consume/**").hasRole("THIRD_CONSUME")
                        .antMatchers("/api/deposit/**").hasRole("THIRD_DEPOSIT")
                        .antMatchers("/api/user/**").hasRole("THIRD_ADMIN")
                        .antMatchers("/api/shop/**").hasRole("THIRD_SHOP")
                        .anyRequest().hasRole("THIRD_COMMON")
                        .and()
                        .csrf().ignoringAntMatchers("/api/**")
//                        .anyRequest().authenticated()
                // 注册 filter
            }
        }

        @Configuration
        class MvcWebSecurityConfigurationAdapter : WebSecurityConfigurerAdapter() {
            @Autowired
            lateinit var dataSource: DataSource
            @Autowired
            lateinit var validateCodeSecurityConfig: ValidateCodeSecurityConfig
            @Autowired
            lateinit var authenticationFailureHandler: AuthenticationFailureHandler
            @Autowired
            lateinit var authenticationSuccessHandler: AuthenticationSuccessHandler


            @Autowired
            lateinit var userDetailsService: OperatorDetailService

            override fun configure(auth: AuthenticationManagerBuilder) {
                auth.authenticationProvider(authenticationProvider())
            }

            @Bean
            fun authenticationProvider(): DaoAuthenticationProvider {
                val authProvider = DaoAuthenticationProvider()
                authProvider.setUserDetailsService(userDetailsService)
                authProvider.setPasswordEncoder(passwordEncoder())
                return authProvider
            }

            @Bean
            fun passwordEncoder(): BCryptPasswordEncoder {
                return BCryptPasswordEncoder()
            }

            @Bean
            fun jdbcTokenImplement(): JdbcTokenRepositoryImpl {
                return JdbcTokenRepositoryImpl().also {
                    it.setDataSource(dataSource)
                }
            }

            override fun configure(http: HttpSecurity) {
                // 设置 Web MVC 应用权限
                http.apply(validateCodeSecurityConfig)
                        .and().csrf()
                        .and()
                        .authorizeRequests()
                        .antMatchers("/login", "/login/form").permitAll()
                        .antMatchers("/static/**").permitAll()
                        .antMatchers("/code/image").permitAll()
                        .antMatchers("/**").hasAnyRole("USER", "ADMIN")
                        .anyRequest().authenticated()
                        .and()
                        .sessionManagement()
                        .sessionCreationPolicy(SessionCreationPolicy.ALWAYS)
                        .and()
                        .formLogin()
                        .loginPage("/login")
                        .loginProcessingUrl("/login/form")
                        .successHandler(authenticationSuccessHandler)
                        .failureHandler(authenticationFailureHandler)
                        .and()
                        .logout()
                        .logoutRequestMatcher(AntPathRequestMatcher("/logout"))
                        .logoutSuccessUrl("/login")
                        .deleteCookies("JSESSIONID")
                        .invalidateHttpSession(true)
                // 设置 Web MVC 应用权限
//                http.apply(validateCodeSecurityConfig)
//                        .and()
//                        .formLogin()
//                        .loginPage("/login")
//                        .loginProcessingUrl("/login/form")
//                        .successHandler(zyAuthenticationSuccessHandler)
//                        .failureHandler(zyAuthenticationFailureHandler)
//                        .and()
//                        .logout()
//                        .logoutRequestMatcher(AntPathRequestMatcher("/logout"))
//                        .logoutSuccessUrl("/login")
//                        .deleteCookies("JSESSIONID")
//                        .invalidateHttpSession(true)
//                        .and()
//                        .userDetailsService(userDetailsService)
//                        .authorizeRequests()
//                        .antMatchers("/login").permitAll()
//                        .antMatchers("/static/**").permitAll()
//                        .antMatchers("/code/image").permitAll()
//                        .anyRequest().authenticated()
//                        .and()
//                        .sessionManagement()
//                        .invalidSessionStrategy(myInvalidSessionStrategy)
//                        .maximumSessions(1)
//                        .sessionRegistry(SessionRegistryImpl())
//                        .maxSessionsPreventsLogin(true)
//                        .and()
//                        .and()
//                        .headers().frameOptions().disable()
            }
        }
    }
}
